using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Collisions;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Dynamics.Springs;
using FarseerGames.FarseerPhysics.Factories;
using StickyFingaz.Controls;
using StickyFingaz.Level;
using StickyFingaz.Handlers;
using StickyFingaz.GameStates;
using FarseerGames.AdvancedSamples;

namespace StickyFingaz
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        #region Properties
        PhysicsSimulator physicsSimulator;
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        TextSpriteHandler textSpriteHandler;
        SoundHandler soundHandler;
        private int timeToNext;
        private Random random;
        private GameState state;
        private Level.Level level;

        public Random Random
        {
            get { return random; }
        }

        public Level.Level CurrentLevel 
        { 
            get { return level; }
            set { level = value; }
        }

        #endregion
#if PHYSICS_DEBUG
        private PhysicsSimulatorView view;
#endif

        public Game1()
        {
            physicsSimulator = new PhysicsSimulator(new Vector2(0.0f, 0.0f));

            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            random = new Random((int)DateTime.Now.Ticks);
            timeToNext = random.Next(5000, 10000);
        }

        public PhysicsSimulator PhysicsSimulator
        {
            get { return physicsSimulator; }
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
            textSpriteHandler = TextSpriteHandler.GetInstance(this);

#if DEBUG
            graphics.PreferredBackBufferHeight = 600;
            graphics.PreferredBackBufferWidth = 800;
            graphics.PreferMultiSampling = true;            
#else
            graphics.PreferredBackBufferHeight = 600;
            graphics.PreferredBackBufferWidth = 800;
            graphics.PreferMultiSampling = true;
            graphics.IsFullScreen = true;
#endif
            graphics.ApplyChanges();

            //ChangeState(new GameOverState(this, false));
            ChangeState(new TitleGameState(this));
            //ChangeState(new MainGameState(this));
}

        public void ChangeState(GameState newstate)
        {
            if(this.state != null)
                this.state.Leave();

            newstate.Enter(this.state);
            this.state = newstate;
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            soundHandler = new SoundHandler();
            // TODO: use this.Content to load your game content here

#if PHYSICS_DEBUG
            view = new PhysicsSimulatorView(physicsSimulator);
            view.LoadContent(graphics.GraphicsDevice, Content);
            view.EnableEdgeView = true;
#endif
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            state.Update(gameTime);
            physicsSimulator.Update(gameTime.ElapsedGameTime.Milliseconds / 1000.0f );
            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            
            state.Draw(gameTime);
            textSpriteHandler.Draw(gameTime);

#if PHYSICS_DEBUG

            spriteBatch.Begin();
            view.Draw(spriteBatch);
            spriteBatch.End();
#endif

            base.Draw(gameTime);
        }
    }
}
