﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using StickyFingaz.Level;
using Microsoft.Xna.Framework.Audio;
using StickyFingaz.Handlers;

namespace StickyFingaz.GameStates
{
    public class MainGameState : GameState
    {
        #region Properties
        private enum MainGameStateState
        {
            MGSS_HAPPY, MGSS_FADE, MGSS_SCARY
        };        
        private Player player;
        private Level.Level level;
        private int timeElapsed, timeToNext, elapsedAudioMillis;
        private Cue currentMusic;
        private MainGameStateState state;
        #endregion

        public MainGameState(Game1 game)
            : base(game)
        { 
        }

        private void ChangeState(MainGameStateState newstate)
        {
            timeElapsed = 0;
            if (currentMusic != null)
                currentMusic.Stop(AudioStopOptions.Immediate);

            switch (newstate)
            {
                case MainGameStateState.MGSS_HAPPY:
                    this.game.CurrentLevel.SpawnEnemies();
                    this.game.CurrentLevel.SetBackgroundOverlay(false);
                    currentMusic = SoundHandler.GetInstance().Play("happyloop");

                    if(this.game.CurrentLevel != null)
                        this.game.CurrentLevel.IsInsanity = false;
                    break;
                case MainGameStateState.MGSS_FADE:
                    currentMusic = SoundHandler.GetInstance().Play("grunt");
                    break;
                case MainGameStateState.MGSS_SCARY:
                    this.game.CurrentLevel.SetBackgroundOverlay(true);
                    currentMusic = SoundHandler.GetInstance().Play("scaryloop");

                    if (this.game.CurrentLevel != null)
                        this.game.CurrentLevel.IsInsanity = true;
                    
                    break;
            }
          
            this.state = newstate;
        }

        public override void Enter(GameState previousState)
        {
            level = game.CurrentLevel = new Level.Level("content\\level1.txt", this.game);
            player = new Player(this.game, level.PlayerSpawnPoint, 1.0f, MathHelper.Pi / 4.0f);
            level.player = player;

            ChangeState(MainGameStateState.MGSS_HAPPY);
        }

        public override void Leave()
        {
            TextSpriteHandler.GetInstance(this.game).Clear();
            TextSpriteHandler.GetInstance(this.game).scrollAmount = Vector2.Zero;

            if (currentMusic != null)
                currentMusic.Stop(AudioStopOptions.Immediate);
        }

        public override void Draw(GameTime gameTime)
        {
            player.Draw(gameTime);
            game.CurrentLevel.Draw(gameTime);
        }

        public override void Update(GameTime gameTime)
        {
            timeElapsed += gameTime.ElapsedGameTime.Milliseconds;

            switch (state)
            {
                case MainGameStateState.MGSS_HAPPY:
                    if (timeElapsed >= 28000)
                    {                        
                       ChangeState(MainGameStateState.MGSS_FADE);
                    }
                    break;
                case MainGameStateState.MGSS_FADE:
                    if (timeElapsed >= 2000)
                    {
                        ChangeState(MainGameStateState.MGSS_SCARY);
                    }
                    break;
                case MainGameStateState.MGSS_SCARY:
                    if (timeElapsed >= 5000)
                    {
                        ChangeState(MainGameStateState.MGSS_HAPPY);
                    }
                    break;
            }

            player.Update(gameTime);
            level.Update(gameTime);
        }

        private void UpdateAudio(GameTime gameTime)
        {
            elapsedAudioMillis += gameTime.ElapsedGameTime.Milliseconds;

            if (elapsedAudioMillis >= timeToNext)
            {
                Random random = this.game.Random;
                SoundHandler soundHandler = SoundHandler.GetInstance();

                timeToNext = random.Next(5000, 10000);
                elapsedAudioMillis = 0;

                switch (random.Next(8))
                {
                    case 0:
                        soundHandler.Play("sf voice blaoahab");
                        break;
                    case 1:
                        soundHandler.Play("sf voice brr whla wha");
                        break;
                    case 2:
                        soundHandler.Play("sf voice dingle wab");
                        break;
                    case 3:
                        soundHandler.Play("sf voice doub");
                        break;
                    case 4:
                        soundHandler.Play("sf voice ghabb");
                        break;
                    case 5:
                        soundHandler.Play("sf voice kha grr ka ruuj");
                        break;
                    case 6:
                        soundHandler.Play("sf voice low wollie bolle lie la");
                        break;
                    case 7:
                        soundHandler.Play("sf voice wlalalielah");
                        break;
                }
            } 
        }
    }
}
