using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework;
using System.Collections;

namespace StickyFingaz.Handlers
{
  public class SoundHandler
  {
    private AudioEngine engine;
    private WaveBank wavebank;
    private SoundBank soundbank;
    private Hashtable musicPlaying;
    private Queue<Cue> cues;
    private static SoundHandler singleton;

    public SoundHandler()
    {
      engine = new AudioEngine("Content\\Audio\\Sticky.xgs");
      wavebank = new WaveBank(engine, "Content\\Audio\\Wave Bank.xwb");
      soundbank = new SoundBank(engine, "Content\\Audio\\Sound Bank.xsb");
      musicPlaying = new Hashtable();
      cues = new Queue<Cue>();
    }

    public static SoundHandler GetInstance()
    {
        if (singleton == null)
        {
            singleton = new SoundHandler();
        }

        return singleton;
    }

    public Cue Play(string name, bool isMusic)
    {
      Cue cue = null;

      if (isMusic && musicPlaying[name] == null)
      {
        cue = soundbank.GetCue(name);
        musicPlaying.Add(name, cue);
      }
      else if (!isMusic)
      {
        cue = soundbank.GetCue(name);
        cues.Enqueue(cue);
      }

      if (cue != null)
        cue.Play();

      return cue;
    }

    public Cue Play(string name)
    {
      return Play(name, false);
    }

    public void Stop(string name)
    {
      if (musicPlaying[name] != null)
      {
        Cue c = (Cue)musicPlaying[name];
        c.Stop(AudioStopOptions.Immediate);
        c.Dispose();
        musicPlaying.Remove(name);
      }
    }

    public void Update(GameTime gameTime)
    {
      engine.Update();

      if (cues.Count > 0 && cues.Peek().IsStopped)
      {
        Cue c = cues.Dequeue();
        c.Dispose();
      }
    }
  }
}
