using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using StickyFingaz.Handlers;

namespace StickyFingaz.Handlers
{
  public class TextSpriteHandler
  {
    #region Privates
    private Game game;
    private List<Text> texts;
    private List<Sprite> sprites;
    private SpriteBatch spriteBatch;
    private SpriteFont font;
    private static TextSpriteHandler instance;
    #endregion

    public Vector2 scrollAmount;

    public TextSpriteHandler(Game game)
    {
      this.game = game;
      this.texts = new List<Text>();
      this.sprites = new List<Sprite>();
      this.spriteBatch = new SpriteBatch(game.GraphicsDevice);

      this.font = game.Content.Load<SpriteFont>("font"); 
    }

    public void AddText(Text text)
    {
      lock (this)
      {
        this.texts.Add(text);
      }
    }

    public void Clear()
    {
        texts.Clear();
        sprites.Clear();
    }

    public Text CreateText(string text, Vector2 position, Color color)
    {
      lock (this)
      {
        Text t = new Text(text, position, color);
        texts.Add(t);

        return t;
      }
    }

    public Sprite CreateSprite(Texture2D texture, Vector2 position)
    {
        return CreateSprite(texture, position, true);
    }
      
    public Sprite CreateSprite(Texture2D texture, Vector2 position, bool isScrolling)
    {
      lock (this)
      {
        Sprite s = new Sprite(texture, position);
        s.IsScrolling = isScrolling;
        this.sprites.Add(s);

        return s;
      }
    }

    public void Draw(GameTime gameTime)
    {
      lock (this)
      {
        this.spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.FrontToBack, SaveStateMode.None);

        // Draw sprites
        foreach (Sprite s in this.sprites)
        {
          if (!s.IsVisible)
            continue;

          switch (s.Type)
          {
              case SpriteType.NORMAL:
                  {
                      Vector2 position = s.Position;
                      if (s.IsScrolling)
                      {
                          position += scrollAmount;
                      }

                      this.spriteBatch.Draw(s.Texture, position, null, s.Color, s.Rotation, s.Origin, s.Scale, SpriteEffects.None, s.Depth);
                  }
                  break;
              case SpriteType.DESTINATION_RECTANGLE:
                  {
                      Rectangle rect = s.Rectangle;
                      rect.X += (int)scrollAmount.X;
                      rect.Y += (int)scrollAmount.Y;

                      this.spriteBatch.Draw(s.Texture, rect, null, s.Color, s.Rotation, s.Origin, SpriteEffects.None, s.Depth);
                  }
                  break;
          }
        }

        // Draw 2d text
        foreach (Text t in this.texts)
        {
          if (!t.IsVisible)
            continue;

          this.spriteBatch.DrawString(this.font, t.Value, t.Position, t.Color, 0, Vector2.Zero, t.Scale,
            SpriteEffects.None, 1.0f);
        }

        this.spriteBatch.End();
      }
    }

    public static TextSpriteHandler GetInstance(Game game)
    {
        if (instance == null)
        {
            instance = new TextSpriteHandler(game);
        }
        return instance;
    }

    public SpriteBatch GetSpriteBatch()
    {
      return this.spriteBatch;
    }

    public void RemoveSprite(Sprite s)
    {
      this.sprites.Remove(s);
    }

    public void RemoveText(Text t)
    {
      t.Dispose();
      this.texts.Remove(t);
    }
  }
}
