﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;
using StickyFingaz.Handlers;
using FarseerGames.FarseerPhysics.Factories;

namespace StickyFingaz.Level.Enemies
{
    public enum EnemyType
    {
        ET_PLAYER_ONE, ET_PLAYER_TWO, ET_NEUTRAL
    };

    public abstract class Enemy : GameObject
    {
        #region Properties
        protected EnemyType type;
        protected Body physicsBody;
        protected Geom physicsGeometry;
        protected EnemySpawnPoint spawnPoint;
        protected Vector2 force;

        public Vector2 Force
        {
            get { return force; }
            set { force = value; }
        }

        public EnemyType Type
        {
            get { return type; }
            set { type = value; }
        }
        #endregion

        public Enemy(Game1 game, EnemySpawnPoint spawnPoint, Vector2 position, EnemyType type)
            : base(game, position, 1, 0)
        {
            this.type = type;
            this.spawnPoint = spawnPoint;
        }

        public virtual void Remove()
        {
        }

        public bool IsHit(Vector2 position, bool isLeftPlayer)
        {
            return (this.position - position).Length() <= 64;                    
        }

        public virtual void SetScale(Vector2 value)
        {
            
        }

        public override void Draw(GameTime gameTime)
        {
        }

        public override void Update(GameTime gameTime)
        {
            physicsBody.ClearForce();

            if (force != Vector2.Zero)
            {
                physicsBody.ApplyForce(ref force);                
            }
        }
    }
}
