﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using StickyFingaz.Handlers;
using Microsoft.Xna.Framework.Graphics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;
using FarseerGames.FarseerPhysics.Factories;

namespace StickyFingaz.Level.Enemies
{
    public class EnemyHound : Enemy
    {
        private Sprite sprite1;
        private Sprite sprite2;
        private int timeElapsed;
        private int timeSinceSwitch;

        public EnemyHound(Game1 game, EnemySpawnPoint spawnPoint, Vector2 position, Vector2 scale_, EnemyType type)
            : base(game, spawnPoint, position, type)
        {
            switch (type)
            {
                case EnemyType.ET_PLAYER_ONE:
                    sprite1 = TextSpriteHandler.GetInstance(game).CreateSprite(
                        this.Game.Content.Load<Texture2D>("EnemyIdleBlue"),
                        position);
                    sprite2 = TextSpriteHandler.GetInstance(game).CreateSprite(
                        this.Game.Content.Load<Texture2D>("EnemyMouthopenBlue"),
                        position);
                    break;
                case EnemyType.ET_PLAYER_TWO:
                    sprite1 = TextSpriteHandler.GetInstance(game).CreateSprite(
                        this.Game.Content.Load<Texture2D>("EnemyIdleRed"),
                        position);
                    sprite2 = TextSpriteHandler.GetInstance(game).CreateSprite(
                        this.Game.Content.Load<Texture2D>("EnemyMouthopenRed"),
                        position);
                    break;
                case EnemyType.ET_NEUTRAL:
                    break;
            }

            if (sprite1 != null)
            {
                sprite1.Scale = sprite2.Scale = scale_;
                sprite1.Depth = sprite2.Depth = 1.0f;
                sprite1.Origin = sprite2.Origin = new Vector2(256, 256);

                sprite2.IsVisible = false;
            }

            // use smaller scale for collsion on enemies
            scale_ *= 0.75f;
            //use the body factory to create the physics body
            physicsBody = BodyFactory.Instance.CreateRectangleBody(((Game1)game).PhysicsSimulator, sprite1.Texture.Width * scale_.X, sprite1.Texture.Height * scale_.Y, 1);
            physicsBody.IsStatic = false;
            physicsGeometry = GeomFactory.Instance.CreateCircleGeom(((Game1)game).PhysicsSimulator, physicsBody, 55.0f, 10, 0);
                //GeomFactory.Instance.CreateRectangleGeom(((Game1)game).PhysicsSimulator, physicsBody, sprite1.Texture.Width * scale_.X, sprite1.Texture.Height * scale_.Y, 100);
            physicsGeometry.RestitutionCoefficient = .5f;
            physicsGeometry.FrictionCoefficient = .5f;
            physicsGeometry.Tag = this;
            physicsBody.Position = position;

            physicsGeometry.CollisionGroup = 0;
        }

        public override void Remove()
        {
            base.Remove();
            physicsBody.Dispose();
            TextSpriteHandler handler = TextSpriteHandler.GetInstance(game);

            handler.RemoveSprite(this.sprite1);
            handler.RemoveSprite(this.sprite2);       
        }

        public override void SetScale(Vector2 value)
        {
            sprite1.Scale = value;
            sprite2.Scale = value;
        }

        private void UpdateAnimation(GameTime gameTime)
        {
            timeElapsed += gameTime.ElapsedGameTime.Milliseconds;
            if (timeElapsed > 1000)
            {
                timeSinceSwitch += gameTime.ElapsedGameTime.Milliseconds;
                if (timeSinceSwitch > 250)
                {
                    timeSinceSwitch = 0;

                    if (sprite1.IsVisible)
                    {
                        sprite1.IsVisible = false;
                        sprite2.IsVisible = true;
                    }
                    else
                    {
                        sprite1.IsVisible = true;
                        sprite2.IsVisible = false;
                    }
                }
            }
            if (timeElapsed > 2000)
            {
                timeSinceSwitch = timeElapsed = 0;                
            }
        }

        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);

            position = physicsBody.Position;

            sprite1.Position = physicsBody.Position;
            sprite1.Rotation = physicsBody.Rotation;
            sprite2.Position = physicsBody.Position;
            sprite2.Rotation = physicsBody.Rotation;

            UpdateAnimation(gameTime);
        }
    }
}
