﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using StickyFingaz.Handlers;
using Microsoft.Xna.Framework.Graphics;
using StickyFingaz.Level.Enemies;

namespace StickyFingaz.Level 
{
    public struct EnemyHitInformation
    {
        public bool isHit;
        public bool isOwnColor;
    };

    public class Level : DrawableGameComponent
    {
        #region Properties
        private List<RigidObject> rigids;
        private List<GameObject> gameObjects;
        private List<BackgroundObject> backgrounds;
        private List<Enemy> enemies;
        private Game1 game;
        private float levelWidth;
        private float levelHeight;
        private float timeExisting;
        private Sprite bgLayer0;
        private Sprite bgLayer1;
        private Sprite bgLayer3;

        private Vector2 playerSpawnPoint;
        private bool isInsanity;

        public Vector2 PlayerSpawnPoint 
        {
            get
            {
                return playerSpawnPoint;
            }
        }

        public bool IsInsanity
        {
            set { isInsanity = value; }
            get { return isInsanity; }
        }
        #endregion

        public Player player;

        public Level(string fileName, Game1 game_) : base(game_)
        {
            playerSpawnPoint = new Vector2(400,300);
            game = game_;
            timeExisting = 0;
            
            bgLayer0 = TextSpriteHandler.GetInstance(game).CreateSprite(
                game.Content.Load<Texture2D>("Background_Layer0"),
                Vector2.Zero,
                false);
            bgLayer0.Scale = new Vector2(1.0f, 1.0f);
            bgLayer0.Depth = 0.0f;

            bgLayer1 = TextSpriteHandler.GetInstance(game).CreateSprite(
                game.Content.Load<Texture2D>("Background_Layer1_Frame1"),
                Vector2.Zero,
                false);
            bgLayer1.Scale = new Vector2(1.0f, 1.0f);
            bgLayer1.Depth = 0.001f;

            bgLayer3 = TextSpriteHandler.GetInstance(game).CreateSprite(
                game.Content.Load<Texture2D>("Background_Layer3"),
                Vector2.Zero,
                false);
            bgLayer3.Scale = new Vector2(1.0f, 1.0f);
            bgLayer3.Depth = 0.003f;

            enemies = new List<Enemy>();
            rigids = new List<RigidObject>();
            gameObjects = new List<GameObject>();
            readLevel(fileName);
        }

        public void AddEnemy(Enemy enemy)
        {
            enemies.Add(enemy);
        }

        public bool IsEnemyHit(Vector2 position, bool isLeftPlayer, out EnemyHitInformation info)
        {
            int index = -1;
            int i=0;
            bool result = false;
            info = new EnemyHitInformation();
            Enemy enemyToBeRemoved = null;
            SoundHandler soundHandler = SoundHandler.GetInstance();

            foreach (Enemy enemy in enemies)
            {
                if(enemy.IsHit(position, isLeftPlayer))
                {
                    info.isHit = true;

                    if (enemy.Type == EnemyType.ET_NEUTRAL)
                    {
                        info.isOwnColor = true;
                    }
                    else if (isLeftPlayer)
                    {                        
                        info.isOwnColor = isLeftPlayer &&
                                          enemy.Type == EnemyType.ET_PLAYER_ONE;
                    }
                    else
                    {                        
                        info.isOwnColor = !isLeftPlayer &&
                                            enemy.Type == EnemyType.ET_PLAYER_TWO;
                    }

                    if (info.isOwnColor)
                    {
                        soundHandler.Play("hitwrongenemy");
                    }
                    else
                    {
                        soundHandler.Play("hitcorrectenemy");
                    }

                    enemyToBeRemoved = enemy;
                    enemy.Remove();
                    index = i;
                    result = true;
                    break;
                }
                i++;
            }

            if (index != -1)
            {
                enemyToBeRemoved = null;
                enemies.RemoveAt(i);
            }


            return result;
        }

        public override void Draw(GameTime gameTime)
        {
            foreach (RigidObject rig in rigids)
            {
                rig.Draw(gameTime);
            }
        }

        public void SpawnEnemies()
        {
            foreach (Enemy enemy in enemies)
            {
                enemy.Remove();
            }
            enemies.Clear();
            
            foreach (GameObject obj in gameObjects)
            {
                if (obj.GetType() == typeof(EnemySpawnPoint))
                {
                    //((EnemySpawnPoint)obj).Reset();
                    ((EnemySpawnPoint)obj).Spawn();
                }
            }
        }

        public override void Update(GameTime gameTime)
        {
            timeExisting += gameTime.ElapsedGameTime.Milliseconds / 1000.0f;
            float loopTime = 0.5f;
            float fragment = timeExisting % loopTime;
            if (fragment < loopTime / 4)
            {
                bgLayer1.Texture = game.Content.Load<Texture2D>("Background_Layer1_Frame1");
            }
            else if (fragment < loopTime / 2)
            {
                bgLayer1.Texture = game.Content.Load<Texture2D>("Background_Layer1_Frame2");
            }
            else if (fragment < loopTime / 4 * 3)
            {
                bgLayer1.Texture = game.Content.Load<Texture2D>("Background_Layer1_Frame3");
            }
            else if (fragment <= loopTime)
            {
                bgLayer1.Texture = game.Content.Load<Texture2D>("Background_Layer1_Frame4");
            }

            foreach (GameObject obj in gameObjects)
            {
                obj.Update(gameTime);
            }

            foreach (RigidObject rig in rigids)
            {
                rig.Update(gameTime);
            }

            for (int i = 0; i < enemies.Count; i++)
            {
                Enemy enemy = enemies[i];
                if (enemy == null)
                {
                    enemies.RemoveAt(i);
                    i--;
                }
                
                if (isInsanity && player != null)
                {
                    Vector2 diff = -(enemy.Position - player.Position);
                    diff.Normalize();

                    enemy.Type = EnemyType.ET_NEUTRAL;
                    enemy.Force = diff * 50;
                }

                enemy.Update(gameTime);
            }
        }

        private void readLevel(string fileName)
        {
            TextReader tr = new StreamReader(fileName);

            // read a line of text
            string line;
            int lineCount = 0;
            int colCount = 0;
            while ((line = tr.ReadLine()) != null)
            {
                int curCol = 0;
                foreach (char c in line)
                {
                    if (c != '-')
                    {
                        createLevelObject(c, curCol, lineCount);
                    }
                    curCol++;
                }
                if (curCol > colCount)
                {
                    colCount = curCol;
                }
                lineCount++;
            }

            levelWidth = colCount * 64;
            levelHeight = lineCount * 64;

           tr.Close();
        }
        
        public void SetBackgroundOverlay(bool isInsane)
        {
            if(isInsane)
            {
                bgLayer0.Color = Color.Red;
                bgLayer1.Color = Color.Red;
                bgLayer3.Color = Color.Red;
            }
            else
            {
                bgLayer0.Color = Color.White;
                bgLayer1.Color = Color.White;
                bgLayer3.Color = Color.White;
            }
        }

        private void createLevelObject(char id, int column, int row)
        {
            switch (id)
            {
                case 'A':
                    RigidObject newWall = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "A. Buis_LeftRight");
                    rigids.Add(newWall);
                    break;
                case 'B':
                    RigidObject newWall2 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "B. Buis_TopBottom");
                    rigids.Add(newWall2);
                    break;
                case 'C':
                    RigidObject newWall3 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "C. Buis_LeftTop");
                    rigids.Add(newWall3);
                    break;
                case 'D':
                    RigidObject newWall4 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "D. Buis_TopRight");
                    rigids.Add(newWall4);
                    break;
                case 'E':
                    RigidObject newWall5 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "E. Buis_LeftBottom");
                    rigids.Add(newWall5);
                    break;
                case 'F':
                    RigidObject newWall6 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "F. Buis_RightBottom");
                    rigids.Add(newWall6);
                    break;
                case 'G':
                    RigidObject newWall7 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "G. Buis2_LeftRight");
                    rigids.Add(newWall7);
                    break;
                case 'H':
                    RigidObject newWall8 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "H. Buis2_TopBottom");
                    rigids.Add(newWall8);
                    break;
                case 'I':
                    RigidObject newWall9 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "I. Buis2_LeftTop");
                    rigids.Add(newWall9);
                    break;
                case 'J':
                    RigidObject newWall10 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "J. Buis2_TopRight");
                    rigids.Add(newWall10);
                    break;
                case 'K':
                    RigidObject newWall11 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "K. Buis2_LeftBottom");
                    rigids.Add(newWall11);
                    break;
                case 'L':
                    RigidObject newWall12 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "L. Buis2_RightBottom");
                    rigids.Add(newWall12);
                    break;
                case 'M':
                    RigidObject newWall13 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "M. Muur");
                    rigids.Add(newWall13);
                    break;
                case 'N':
                    RigidObject newWall14 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "N. Muur_top");
                    rigids.Add(newWall14);
                    break;
                case 'O':
                    RigidObject newWall15 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "O. Muur_metBuis");
                    rigids.Add(newWall15);
                    break;

                case 'P':
                    RigidObject newWall16 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "P. Buis_OpenBottom");
                    rigids.Add(newWall16);
                    break;
                case 'Q':
                    RigidObject newWall17 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "Q. Buis_OpenRight");
                    rigids.Add(newWall17);
                    break;
                case 'R':
                    RigidObject newWall100 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "R. Buis_OpenTop");
                    rigids.Add(newWall100);
                    break;
                case 'S':
                    RigidObject newWall18 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "S. Buis2_OpenRight");
                    rigids.Add(newWall18);
                    break;
                case 'V':
                    RigidObject newWall19 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "V. Buis2_OpenTop");
                    rigids.Add(newWall19);
                    break;
                case 'W':
                    RigidObject newWall20 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "W. Buis2_OpenLeft");
                    rigids.Add(newWall20);
                    break;
                case 'X':
                    RigidObject newWall21 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "X. MuurHoek1");
                    rigids.Add(newWall21);
                    break;
                case 'Y':
                    RigidObject newWall22 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "Y. MuurHoek2");
                    rigids.Add(newWall22);
                    break;
                case 'Z':
                    RigidObject newWall23 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "Z. MuurZij1");
                    rigids.Add(newWall23);
                    break;
                case '1':
                    RigidObject newWall24 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "1. MuurZij2");
                    rigids.Add(newWall24);
                    break;
                case '2':
                    RigidObject newWall25 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "2. MuurHoek3");
                    rigids.Add(newWall25);
                    break;
                case '3':
                    RigidObject newWall26 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "3. MuurHoek4");
                    rigids.Add(newWall26);
                    break;
                case '4':
                    RigidObject newWall27 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "4. MuurZij4");
                    rigids.Add(newWall27);
                    break;
                case '5':
                    RigidObject newWall28 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "5. MuurHoek5");
                    rigids.Add(newWall28);
                    break;
                case '6':
                    RigidObject newWall29 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "6. Muur_Hoek6");
                    rigids.Add(newWall29);
                    break;
                case '7':
                    RigidObject newWall30 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "7. Muur_Hoek7");
                    rigids.Add(newWall30);
                    break;
                case '8':
                    RigidObject newWall31 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "8. Muur_Hoek8");
                    rigids.Add(newWall31);
                    break;
                case '9':
                    RigidObject newWall32 = new RigidObject(game, new Vector2(column * 64, row * 64), new Vector2(64.0f / 75.0f), 0, "9. Prop1");
                    rigids.Add(newWall32);
                    break;

                case '!':
                    EnemySpawnPoint spawnpoint1 = new EnemySpawnPoint(
                        game, new Vector2(column * 64, row * 64), this, EnemyType.ET_PLAYER_ONE);
                    gameObjects.Add(spawnpoint1);
                    break;
                case '?':
                    EnemySpawnPoint spawnpoint2 = new EnemySpawnPoint(
                        game, new Vector2(column * 64, row * 64), this, EnemyType.ET_PLAYER_TWO);
                    gameObjects.Add(spawnpoint2);
                    break;
                case '@':
                    playerSpawnPoint = new Vector2(column * 64, row * 64);
                    break;
            }
        }
    }
}
