﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using StickyFingaz.Controls;
using StickyFingaz.Handlers;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;
using FarseerGames.FarseerPhysics.Factories;
using Microsoft.Xna.Framework.Input;
using StickyFingaz.Level.Enemies;
using StickyFingaz.GameStates;

namespace StickyFingaz.Level
{
    public class Player : GameObject
    {
        private Cursor cursorPlayer1, cursorPlayer2;
        private List<PlayerArm> p1Arms;
        private List<PlayerArm> p2Arms;
        private Dictionary<int, Vector2> forces;        
        private Sprite sprite;
        private Sprite backSprite;
        private int nextForceIndex;
        private Body physicsBody;
        private Geom physicsGeometry;
        private SoundHandler soundHandler;
        private TextSpriteHandler textSpriteHandler;
        private Text textScorePlayer1, textScorePlayer2;
        private int scorePlayer1, scorePlayer2;

        private int winScore;
        float lastWallHitTime;

        private int cooldownPlayer1, cooldownPlayer2;

        public Player(Game1 game, Vector2 position_, float scale_, float rotation_)
            : base(game, position_, scale_, rotation_)
        {
            soundHandler = SoundHandler.GetInstance();
            nextForceIndex = 0;
            lastWallHitTime = 0;
            winScore = 5;
            scorePlayer1 = 0;
            scorePlayer2 = 0;
            
#if DEBUG
            cursorPlayer1 = new MouseCursor(game, false);
            cursorPlayer2 = new MouseCursor(game, true);
#else
            cursorPlayer1 = new XboxCursor(game, PlayerIndex.One);
            cursorPlayer2 = new XboxCursor(game, PlayerIndex.Two);
#endif

            cooldownPlayer1 = cooldownPlayer2 = 0;

            if (GamePad.GetState(PlayerIndex.One).IsConnected &&
                GamePad.GetState(PlayerIndex.Two).IsConnected)
            {
                cursorPlayer1 = new XboxCursor(game, PlayerIndex.One);
                cursorPlayer2 = new XboxCursor(game, PlayerIndex.Two);
            }
            else
            {
                cursorPlayer1 = new MouseCursor(game, false);
                cursorPlayer2 = new MouseCursor(game, true);
            }

            cursorPlayer1.Sprite = TextSpriteHandler.GetInstance(game).CreateSprite(
                game.Content.Load<Texture2D>("CursorBlue"),
                Vector2.Zero,
                false);
            cursorPlayer2.Sprite = TextSpriteHandler.GetInstance(game).CreateSprite(
                game.Content.Load<Texture2D>("CursorRed"),
                Vector2.Zero,
                false);
            cursorPlayer1.Sprite.Scale = cursorPlayer2.Sprite.Scale = new Vector2(0.5f);
            cursorPlayer1.Sprite.Depth = cursorPlayer2.Sprite.Depth = 1.0f;
            cursorPlayer1.Sprite.Origin = cursorPlayer2.Sprite.Origin =
                new Vector2(64, 64);            

            p1Arms = new List<PlayerArm>();
            p2Arms = new List<PlayerArm>();

            forces = new Dictionary<int, Vector2>();
            sprite = TextSpriteHandler.GetInstance(game).CreateSprite(
                game.Content.Load<Texture2D>("MainCharacterIdle"),
                Vector2.Zero,
                true);
            sprite.Scale = new Vector2(1.0f / 8.0f, 1.0f / 8.0f);
            sprite.Origin = new Vector2(512, 512);
            sprite.Depth = 1.0f;
            

            backSprite = TextSpriteHandler.GetInstance(game).CreateSprite(
                game.Content.Load<Texture2D>("MainCharacterBehindarms"),
                Vector2.Zero,
                true);
            backSprite.Scale = new Vector2(1.0f / 8.0f, 1.0f / 8.0f);
            backSprite.Origin = new Vector2(512, 512);
            backSprite.Depth = 0.2f;


            physicsBody = BodyFactory.Instance.CreateRectangleBody(((Game1)game).PhysicsSimulator, 110, 110, 5);
            physicsBody.Position = position;
            physicsBody.LinearDragCoefficient = 4.0f;

            physicsGeometry = GeomFactory.Instance.CreateCircleGeom(((Game1)game).PhysicsSimulator, physicsBody, 55.0f, 10,
                                                                        0);
            physicsGeometry.RestitutionCoefficient = .5f;
            physicsGeometry.FrictionCoefficient = .5f;
            physicsGeometry.CollisionGroup = 1;

            physicsGeometry.OnCollision += OnCollision;
            
            
            textSpriteHandler = TextSpriteHandler.GetInstance(this.Game);
            textScorePlayer1 = textSpriteHandler.CreateText("Score: 0", new Vector2(10, 50), Color.Blue);
            textScorePlayer2 = textSpriteHandler.CreateText("Score: 0", new Vector2(600, 50), Color.Red);
        }

        private bool OnCollision(Geom geom1, Geom geom2, ContactList contactList)
        {
            if (geom2.Tag is EnemyHound)
            {
                //SoundHandler.GetInstance().Play("hitspace");
                EnemyHound hound = (EnemyHound)geom2.Tag;
                if (hound != null)
                {
                    switch (hound.Type)
                    {
                        case EnemyType.ET_NEUTRAL:
                            scorePlayer1 = Math.Max(0, scorePlayer1 - 1);
                            scorePlayer2 = Math.Max(0, scorePlayer2 - 1);
                            break;
                        case EnemyType.ET_PLAYER_ONE:
                            scorePlayer1 = Math.Min(winScore, scorePlayer1 + 1);
                            break;
                        case EnemyType.ET_PLAYER_TWO:
                            scorePlayer2 = Math.Min(winScore, scorePlayer2 + 1);
                            break;
                    }
                    hound.Remove();
                }
                return false;
            }
            else if (geom2.Tag is RigidObject)
            {
                if (lastWallHitTime > 2.0f)
                {
                    SoundHandler.GetInstance().Play("sf boingk");
                    lastWallHitTime = 0.0f;
                }
            }
            return true;
        }


        public int addForce(Vector2 force) 
        {
            forces.Add(nextForceIndex, force);
            int index = nextForceIndex;
            nextForceIndex++;
            return index;
        }

        public void removeForce(int forceIndex)
        {
            if (forces.ContainsKey(forceIndex))
            {
                forces.Remove(forceIndex);
            }
        }

        public override void Draw(GameTime gameTime)
        {
        }

        public override void Update(GameTime gameTime)
        {
            if (scorePlayer1 >= winScore)
            {
                this.game.ChangeState(new GameOverState(this.game, false));
                return;
            }
            if (scorePlayer2 >= winScore)
            {
                this.game.ChangeState(new GameOverState(this.game, true));
                return;
            }

            lastWallHitTime += gameTime.ElapsedGameTime.Milliseconds / 1000.0f;

            if (cooldownPlayer1 > 0)
            {
                cooldownPlayer1 -= gameTime.ElapsedGameTime.Milliseconds;
            }
            if (cooldownPlayer2 > 0)
            {
                cooldownPlayer2 -= gameTime.ElapsedGameTime.Milliseconds;
            }


            checkHandCollisions();
            physicsBody.ClearForce();

            foreach (KeyValuePair<int, Vector2> force in forces)
            {
                physicsBody.ApplyForce(force.Value);
            }
            //physicsBody.ApplyForce(physicsBody.GetVelocityAtLocalPoint(new Vector2(0.0f)) *);
            position = physicsBody.Position;
            
            sprite.Position = physicsBody.Position;
            sprite.Rotation = physicsBody.Rotation;
            backSprite.Position = physicsBody.Position;
            backSprite.Rotation = physicsBody.Rotation;
            
            cursorPlayer1.Update(gameTime);
            cursorPlayer2.Update(gameTime);


            for (int i = 0; i < p1Arms.Count; i++)
            {
                if (!p1Arms[i].isAlive)
                {
                    p1Arms[i].clear();
                    p1Arms.RemoveAt(i);
                    i--;
                }
            }
            for (int i = 0; i < p2Arms.Count; i++)
            {
                if (!p2Arms[i].isAlive)
                {
                    p2Arms[i].clear();
                    p2Arms.RemoveAt(i);
                    i--;
                }
            }

            if (cursorPlayer1.IsButtonDown && cooldownPlayer1 <= 0)
            {
                while (p1Arms.Count > 2)
                {
                    p1Arms[0].clear();
                    p1Arms.RemoveAt(0);
                }
             
                Vector2 pos = cursorPlayer1.Position + -textSpriteHandler.scrollAmount;
                PlayerArm arm = new PlayerArm(this.Game, this, pos, false);
                p1Arms.Add(arm);
                EnemyHitInformation hitInfo;

                if (this.Game.CurrentLevel.IsEnemyHit(pos, true, out hitInfo))
                {
                    if (hitInfo.isOwnColor)
                        scorePlayer1--;

                    if (scorePlayer1 < 0)
                        scorePlayer1 = 0;
                }
                else
                {
                    cooldownPlayer1 = 1000;
                    soundHandler.Play("hitspace");
                }
            }

            if (cursorPlayer2.IsButtonDown && cooldownPlayer2 <= 0)
            {
                while (p2Arms.Count > 2)
                {
                    p2Arms[0].clear();
                    p2Arms.RemoveAt(0);
                }
                Vector2 pos = cursorPlayer2.Position + -textSpriteHandler.scrollAmount;
                PlayerArm arm = new PlayerArm(this.Game, this, pos, true);
                p2Arms.Add(arm);
                EnemyHitInformation hitInfo;

                if (this.Game.CurrentLevel.IsEnemyHit(pos, false, out hitInfo))
                {
                    if (hitInfo.isOwnColor)
                        scorePlayer2--;

                    if (scorePlayer2 < 0)
                        scorePlayer2 = 0;
                }
                else
                {
                    cooldownPlayer2 = 1000;
                    soundHandler.Play("hitspace");
                }
            }

            foreach (PlayerArm arm in p1Arms)
            {                
                arm.Update(gameTime);
            }
            foreach (PlayerArm arm in p2Arms)
            {
                arm.Update(gameTime);
            }
            
            UpdateScroll();

            this.textScorePlayer1.Value = string.Format("Score: {0}", this.scorePlayer1);
            this.textScorePlayer2.Value = string.Format("Score: {0}", this.scorePlayer2);
        }


        private void checkHandCollisions()
        {
            for (int i = 0; i < p1Arms.Count; i++)
            {
                for (int j = 0; j < p2Arms.Count; j++)
                {
                    if ((p1Arms[i].handPosition - p2Arms[j].handPosition).Length() < 64 &&
                        !p1Arms[i].slapped &&
                        !p2Arms[j].slapped)
                    {
                        p1Arms[i].doSlapped();
                        p2Arms[j].doSlapped();

                        cooldownPlayer1 = cooldownPlayer2 = 0;

                        SoundHandler.GetInstance().Play("sf hit hands");
                        return;
                    }
                }
            }

        }


        private void UpdateScroll()
        {
            Vector2 midden = new Vector2(
                this.game.GraphicsDevice.PresentationParameters.BackBufferWidth / 2,
                this.game.GraphicsDevice.PresentationParameters.BackBufferHeight / 2);

            Vector2 diff = this.position - midden;

            textSpriteHandler.scrollAmount = -diff;
        }
    }
}
