﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using StickyFingaz.Handlers;
using Microsoft.Xna.Framework.Graphics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Factories;
using FarseerGames.FarseerPhysics.Collisions;

namespace StickyFingaz.Level
{
    public class PlayerArm : GameObject, IDisposable
    {
        private Sprite hand;
        private Sprite armStretch;
        private Sprite armBase;
        private bool wasSlapped;


        private Player player;
        int currentPlayerForce;
        float timeExisting;
        float lifeTime;

        public bool slapped 
        {
            get
            {
                return wasSlapped;
            }
        }

        public Vector2  handPosition
        {
            get
            {
                return hand.Position;
            }
        }


        Vector2 originalPosition;

        public Boolean isAlive 
        {
            get
            {
                return lifeTime - timeExisting > 0;
            }
        }

        public PlayerArm(Game1 game, Player player_, Vector2 position, Boolean player2)
            : base(game,
                position,
                1, 0)                
        {
            timeExisting = 0.0f;
            lifeTime = 5.0f;

            wasSlapped = false;
            
            string handTexture = player2 ? "ArmRedHand" : "ArmBlueHand";
            string stretchTexture = player2 ? "ArmRedMid" : "ArmBlueMid";
            string armBaseTexture = player2 ? "ArmRedStart" : "ArmBlueStart";

            originalPosition = position;
            currentPlayerForce = -1;
            hand = TextSpriteHandler.GetInstance(game).CreateSprite(
                game.Content.Load<Texture2D>(handTexture),
                position,
                false);
            hand.Type = SpriteType.DESTINATION_RECTANGLE;
            hand.Origin = new Vector2(128, 256);
            hand.Depth = 0.5f;

            armStretch = TextSpriteHandler.GetInstance(game).CreateSprite(
                game.Content.Load<Texture2D>(stretchTexture),
                position,
                false);
            armStretch.Type = SpriteType.DESTINATION_RECTANGLE;
            armStretch.Origin = new Vector2(128, 512);
            armStretch.Depth = 0.5f;

            armBase = TextSpriteHandler.GetInstance(game).CreateSprite(
                game.Content.Load<Texture2D>(armBaseTexture),
                position,
                false);
            armBase.Type = SpriteType.DESTINATION_RECTANGLE;
            armBase.Origin = new Vector2(128, 256);
            armBase.Depth = 0.5f;

            player = player_;
        }


        public void doSlapped()
        {
            wasSlapped = true;
            timeExisting = Math.Max(timeExisting, lifeTime - 1.0f);
        }


        public override void Draw(GameTime gameTime)
        {
            
        }

        private float DetermineRotation()
        {
            Vector2 diff = this.player.Position - originalPosition;
            diff.Normalize();
            Vector2 up = Vector2.UnitY;
            float angle = (float)Math.Acos(Vector2.Dot(up, diff));
            if (diff.X > 0)
            {
                angle *= -1;
            }
            return angle; 
        }

        private Rectangle DetermineRectangle(float startAt, float endAt, float scale)
        {
            Vector2 diff = originalPosition - this.player.Position;
            
            //float length = diff.Length();

            float startRadius = 35 + startAt;

            //float startLength = playerRadius + startAt;

            diff.Normalize();

            //Vector2 startPos = this.player.Position + diff * startLength;

            return new Rectangle(
                (int)(player.Position.X + diff.X * startRadius),
                (int)(player.Position.Y + diff.Y * startRadius),
                (int)(64 * scale), (int)(endAt - startAt));
        }

        public override void Update(GameTime gameTime)
        {
            timeExisting += gameTime.ElapsedGameTime.Milliseconds / 1000.0f;
            float fadeTime = 2.0f;

            if (lifeTime - timeExisting < fadeTime)
            {
                hand.Color = new Color(1.0f, 1.0f, 1.0f, MathHelper.Max(0.0f, (lifeTime - timeExisting) / fadeTime));
                armStretch.Color = hand.Color;
                armBase.Color = hand.Color;
            }
            if (currentPlayerForce >= 0)
            {
                player.removeForce(currentPlayerForce);
            }
            Vector2 force = (position - player.Position);
            force.Normalize();
            currentPlayerForce = player.addForce(force * 2000);

            float rotation = DetermineRotation();

            Vector2 diff = originalPosition - this.player.Position;
            float scale = 1.0f;
            float length = diff.Length();
            if (length < 128)
            {
                scale = length / 128;
            }

            length = MathHelper.Max(length, 128);

            this.armBase.Rotation = rotation;
            this.armBase.Rectangle = DetermineRectangle(0, 66f * scale, scale);
            
            this.armStretch.Rotation = rotation;
            this.armStretch.Rectangle = DetermineRectangle(64.0f * scale, (length - 30.5f) * scale, scale);

            this.hand.Rotation = rotation;
            this.hand.Rectangle = DetermineRectangle((length - 32) * scale, (length + 32) * scale, scale);


        }

        public void clear()
        {
            TextSpriteHandler.GetInstance(Game).RemoveSprite(armBase);
            TextSpriteHandler.GetInstance(Game).RemoveSprite(armStretch);
            TextSpriteHandler.GetInstance(Game).RemoveSprite(hand);

            if (currentPlayerForce >= 0)
            {
                player.removeForce(currentPlayerForce);
            }
        }

        
    }
}
