﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using StickyFingaz.Handlers;
using FarseerGames.FarseerPhysics.Collisions;
using FarseerGames.FarseerPhysics.Factories;
using Microsoft.Xna.Framework.Graphics;

namespace StickyFingaz.Level
{
    public class RigidObject : GameObject
    {
        private Sprite sprite;
        private Body physicsBody;
        private Geom physicsGeometry;
        
        public RigidObject(Game1 game, 
                           Vector2 position_, Vector2 scale_, float rotation_,
                           string textureName) 
            : base(game, position_,
                           scale_.X, rotation_)
        {
            Texture2D t2d = game.Content.Load<Texture2D>(textureName);
            sprite = TextSpriteHandler.GetInstance(game).CreateSprite(
                t2d,
                Vector2.Zero);
            sprite.Scale = scale_;
            sprite.Depth = 0.1f;
            sprite.Origin = new Vector2(t2d.Width / 2 * scale_.X, t2d.Height / 2 * scale_.Y);
            //use the body factory to create the physics body
            
            physicsBody = BodyFactory.Instance.CreateRectangleBody(((Game1)game).PhysicsSimulator, t2d.Width * scale_.X, t2d.Height * scale_.Y, 1);
            physicsBody.IsStatic = true;
            physicsGeometry = GeomFactory.Instance.CreateRectangleGeom(((Game1)game).PhysicsSimulator, physicsBody, t2d.Width * scale_.X, t2d.Height * scale_.Y, 100);
            physicsGeometry.RestitutionCoefficient = .2f;
            physicsGeometry.FrictionCoefficient = .2f;
            physicsGeometry.Tag = this;
            physicsBody.Position = position;
        }

        public override void Draw(Microsoft.Xna.Framework.GameTime gameTime)
        {
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            sprite.Position = physicsBody.Position;
            sprite.Rotation = physicsBody.Rotation;
        }
    }
}
